import pandas as pd
from sklearn.metrics import silhouette_score
from sklearn import preprocessing
from sklearn.cluster import AgglomerativeClustering
import matplotlib.pyplot as plt
import numpy as np
import math

#ucitavanje skupa
df = pd.read_csv("studenti.csv")

#Atribut INDEKS nece biti koriscen pri klasterovanju jer je jedinstven za svakog studenta.
use_columns = df.columns[1:]
print('Kolone za klasterovanje', use_columns.tolist())

#Za klasterovanje se koriste 3 numericka i 1 kategoricki atribut.
# Da bi kategoricki atribut mogao da se koristi u klasterovanju
# mora biti pretvoren u jedan ili vise numerickih atributa, te ce nad
# njim biti izvrsena binarizacija.
# Funkcija pd.get_dummies vrsi binarizaciju nad svim nenumerickim atributima u objektu klase DataFrame.
# Svaki novi atribut dobijen binarizacijom kategorickog atributa kao prefiks ima ime
# originalnog atributa na osnovu koga je nastao.
df_trans=pd.get_dummies(df[use_columns])

print('Kolone nakon izvrsene binarizacije ', df_trans.columns.tolist())

#normalizacija atributa
x = pd.DataFrame(preprocessing.MinMaxScaler().fit_transform(df_trans))
#dodeljivanje imena kolonama
x.columns = df_trans.columns

# Pri hijerarhijskom klasterovanju bice korisceno euklidsko rastojanje,
# pa ce vrednost 1 u atributima dobijenim na osnovu kategorickog atributa SMER
# biti zamenjena sa sqrt(0.5), da bi svi originalni atributi imali isti uticaj
# pri klasterovanju.
dummy_cols = [col for col in x.columns.tolist() if col.startswith('SMER')]
for col in dummy_cols:
    x[col].replace(1, math.sqrt(0.5), inplace=True )

# primena hijerarhijskog sakupljajuceg klasterovanja za razlicite veze i broj klastera
for link in ['complete', 'average', 'single']:

    # recnik koji ce sadrzati silueta koeficijent za razlicit broj izdvojenih klastera
    silhouette_scores = {'clusters': [], 'score': []}

    for i in range(2,36):
        # primena algoritma za i klastera
        est = AgglomerativeClustering(n_clusters=i, linkage=link)
        est.fit(x)

        #cuvanje podataka o broju klastera i silueta koeficijentu za izvrseno klasterovanje
        #radi grafickog prikaza
        silhouette_scores['clusters'].append(i)
        silhouette_scores['score'].append(silhouette_score(x, est.labels_))

    # prikaz rezultata klasterovanja pomocu grafika broj klastera - senka koeficijent
    # za vezu koriscenu pri hijerarhijskom klasterovanju
    plt.plot(silhouette_scores['clusters'], silhouette_scores['score'], marker='o', label=link)

    #izdvajanje optimalnog broja klastera sa zadatim parametrima pri klasterovanju
    # na osnovu silueta koeficijenta
    best_num_clusters = silhouette_scores['clusters'][np.argmax(silhouette_scores['score'])]

    print('***Veza', link, 'broj klastera sa najvecim senka koeficijentom', best_num_clusters, '***')
    #primena klasterovanja sa optimalnim brojem klastera da bi se
    #izdvojile deskriptivne statistike za svaki klaster
    est = AgglomerativeClustering(n_clusters=i, linkage=link)
    est.fit(x)
    df['label']=est.labels_

    #racuanje i prikaz deskriptivnih statistika za svaki izdvojeni klaster
    for j in range(0, best_num_clusters):
        cluster = df.loc[lambda r: r['label'] == j, :]
        print('Klaster', j)
        print(cluster.describe(include='all'))

# postavljanje teksta za x i y osu
plt.xlabel('Broj klastera')
plt.ylabel('Silueta koeficijent')

# prikaz legende na slici
plt.legend()

#cuvanje slike
plt.savefig('klasterovanje_studenata.png')


#Na osnovu dobijene slike, vidi se da je za sve tri primenjene veze za odredjivanje
# bliskosti dva klastera pri hijerarhijskom klasterovanju, optimalan broj klastera 9 ili 10.

#Za vezbu: uraditi zadatak koriscenjem Menhetn rastojanja.
#Sta je sve potrebno promeniti?